searchData={"items":[{"type":"module","title":"Cascade","doc":"Generate code from templates.","ref":"Cascade.html"},{"type":"function","title":"Cascade.generate/3","doc":"Generate the code associated to the given template `name`.\n\n`root_path` is expected to be the root directory under which the template\nwill be generated. `opts` can be an arbitrary keyword list corresponding\nto the command line options passed to the `mix cascade` task.","ref":"Cascade.html#generate/3"},{"type":"function","title":"Cascade.templates/0","doc":"Returns all available templates.\n\nAll modules implementing the `Cascade.Template` behaviour will be returned.","ref":"Cascade.html#templates/0"},{"type":"module","title":"Cascade.Checks","doc":"Useful generic checks for templates.","ref":"Cascade.Checks.html"},{"type":"function","title":"Cascade.Checks.check_directory_existence!/1","doc":"Ensure that the given directory exists","ref":"Cascade.Checks.html#check_directory_existence!/1"},{"type":"function","title":"Cascade.Checks.check_module_name_availability!/1","doc":"Ensure that the given module name is available.","ref":"Cascade.Checks.html#check_module_name_availability!/1"},{"type":"function","title":"Cascade.Checks.check_module_name_validity!/1","doc":"Ensure that the given `name` is a valid module name.","ref":"Cascade.Checks.html#check_module_name_validity!/1"},{"type":"behaviour","title":"Cascade.Template","doc":"The template generation logic.\n\nEvery `Cascade` template must implement this behaviour. It provides\ncallbacks which affect the template generation logic.","ref":"Cascade.Template.html"},{"type":"callback","title":"Cascade.Template.args_schema/0","doc":"The CLI arguments (if any) schema of the template.\n\nIt should return a valid `CliOptions` arguments schema. These CLI options will\nbe automatically parsed and assigned to the template's bindings when invoked\nfrom the cascade mix task.","ref":"Cascade.Template.html#c:args_schema/0"},{"type":"callback","title":"Examples - Cascade.Template.args_schema/0","doc":"```elixir\ndefmodule MyTemplate do\n  @behaviour Cascade.Template\n\n  @impl true\n  def args_schema do\n    [\n      path: [\n        type: :string,\n        doc: \"The path to store the template to\",\n        required: true\n      ],\n      auth: [\n        type: :boolean,\n        doc: \"Whether to generate authentication logic\",\n        default: false\n      ]\n    ]\n  end\nend\n```","ref":"Cascade.Template.html#c:args_schema/0-examples"},{"type":"callback","title":"Cascade.Template.assets_path/0","doc":"The absolute path to the template.","ref":"Cascade.Template.html#c:assets_path/0"},{"type":"callback","title":"Cascade.Template.destination_path/3","doc":"Returns the destination path for the given template file.\n\nIt expects a file path, the template's config and the user provided CLI arguments and\nis expected to return the destination path for this specific asset. The `asset_path` is\nthe template file's path with respect to the `assets`.\n\nThis will be called for all template files. For example for the following `foo`\ntemplate:\n\n    templates/foo\n    ├── README.md\n    ├── lib\n    │   └── foo.ex\n    └── mix.exs\n\nit will be called for the following asset paths:\n\n- `README.md`\n- `mix.exs`\n- `lib/foo.ex`","ref":"Cascade.Template.html#c:destination_path/3"},{"type":"function","title":"Cascade.Template.generate/3","doc":"Generates a template","ref":"Cascade.Template.html#generate/3"},{"type":"function","title":"Cascade.Template.moduledoc/1","doc":"Gets the moduledoc for the given template `module`.\n\nReturns the moduledoc or `nil`.","ref":"Cascade.Template.html#moduledoc/1"},{"type":"callback","title":"Cascade.Template.name/0","doc":"Returns the name of template.\n\nThe names must be unique, it is advised to use a prefix in order to avoid\ncollisions with other packages.","ref":"Cascade.Template.html#c:name/0"},{"type":"function","title":"Cascade.Template.shortdoc/1","doc":"Gets the shortdoc for the given template `module`.\n\nReturns the shortdoc or `nil`.","ref":"Cascade.Template.html#shortdoc/1"},{"type":"callback","title":"Cascade.Template.validate_cli_opts/1","doc":"Used for custom validation and post-processing of cli arguments.\n\nThis can be used for performing custom validations/casting of cli arguments\nand optionally augment them. It expects the parsed cli arguments and is\nexpected to return `{:ok, opts}` in case of success or `{:error, reason}`\nin case of failure.\n\nAll templates are required to define the expected cli arguments under their\n`config.exs`. The callback will be called in order to validate the user provided\narguments.","ref":"Cascade.Template.html#c:validate_cli_opts/1"},{"type":"callback","title":"Examples - Cascade.Template.validate_cli_opts/1","doc":"Assuming that you have a template that expects a `--path` argument. Since\ngenerator works with absolute paths you can override this callback in order\nto expand the path if it is relative.\n\n```elixir\ndef validate_cli_opts(opts) do\n  path =\n    Keyword.fetch!(opts, :path)\n    |> Path.expand(File.cwd())\n\n  opts = Keyword.put(opts, :path, path)\n\n  {:ok, opts}\nend\n```","ref":"Cascade.Template.html#c:validate_cli_opts/1-examples"},{"type":"module","title":"Cascade.Templates.Template","doc":"Generates a new template.","ref":"Cascade.Templates.Template.html"},{"type":"module","title":"Command line options - Cascade.Templates.Template","doc":"* `--name` (`string`) - Required. The template name.\n* `--assets-path` (`string`) - The assets path with respect to the current working directory.\nThis is where all template assets should be added. By convention\nit defaults to a `templates` folder at the same level as your\n`lib` folder.\n   [default: `templates`]\n* `--templates-path` (`string`) - The path under the `lib` folder where templates are stored. If not\nset defaults to `{app_name}/templates`.","ref":"Cascade.Templates.Template.html#module-command-line-options"},{"type":"task","title":"mix cascade","doc":"Generates code from the given template.\n\nIt expects the template name as argument.\n\n    $ mix cascade template_name [--root PATH] -- [...]\n\nNotice that individual templates may support extra command line\narguments. These must be specified after the `--` separator.\n\nYou can list all available templates by running:\n\n    $ mix cascade.help\n\nYou can get detailed docs for a specific template by running:\n\n    $ mix cascade.help template_name","ref":"Mix.Tasks.Cascade.html"},{"type":"task","title":"Options - mix cascade","doc":"* `--root` (`string`) - The root path under which the template will be generated.\n\n  - If it is not set it defaults to the current working directory.\n  - If it is a relative path is is expanded with respect to the\n  current working directory.\n  - If is is an absolute path it is used unchanged.","ref":"Mix.Tasks.Cascade.html#module-options"},{"type":"task","title":"mix cascade.help","doc":"Lists available templates or prints the documentation for a given template.","ref":"Mix.Tasks.Cascade.Help.html"},{"type":"task","title":"Usage - mix cascade.help","doc":"$ mix cascade.help          - prints all available templates and their short description\n    $ mix cascade.help TEMPLATE - prints full docs for the given template","ref":"Mix.Tasks.Cascade.Help.html#module-usage"},{"type":"extras","title":"Overview","doc":"# Cascade\n\nGenerate code from templates.\n\n`Cascade` provides an easy to use interface for generating code from a set of\ntemplates. It's main features are:\n\n  - *Templates as code* - all template files are evaluated using `EEx`, the actual\n  generation code can be modified by overriding the default implementation.\n  - `mix cascade` - a helper mix task for generating code from a template, it\n  supports parsing of CLI options, and automatically generated help messages from\n  the template file.\n  - `mix cascade.help` - automatically generate help for your custom templates,\n  similar to `mix help`","ref":"readme.html"},{"type":"extras","title":"Usage - Overview","doc":"You can generate a new template under your mix project by running:\n\n```bash\n$ mix cascade template -- --name my_template\n* creating templates/my_template/PLACEHOLDER.md\n* creating lib/cascade/templates/my_template.ex\n```\n\nThis generates two files:\n\n  - The actual template code, which by default is located under\n  `lib/cascade/templates/{template_name}`\n  - A sample template `PLACEHOLDER.md` file under `templates/{template_name}`\n\nIf you now run `mix cascade.help` you will see the newly added template\nin the list of available templates:\n\n```bash\n$ mix cascade.help\nThe following templates are available:\n\n  my_template  # TODO: Add shortdoc\n  template     # Generates \\a new template\n\nRun mix cascade NAME to generate code from the given template\nRun mix cascade.help NAME to see help for a specific template\n```\n\nYou can generate some code using the `my_template` template by running:\n\n```bash\n$ mix cascade my_template\n```\n\nThis will generate all code associated to the given template. In this case it\nwill only generate the `PLACEHOLDER.md` which was added by the `mix cascade`\ncommand.\n\nYou can check the help message of the newly created template by running:\n\n```bash\n$ mix cascade.help my_template\n```","ref":"readme.html#usage"},{"type":"extras","title":"Implementing your template - Overview","doc":"You are now free to implement your actual template logic. You are able to:\n\n  - Add any asset under the the template's assets folder (in our example\n  `templates/my_template`).\n  - The assets can be plain files or `EEx` templates. In the latter case\n  they will be evaluated during generation.\n  - Define a set of CLI arguments that your template expects. `mix cascade`\n  will validate these options automatically. Notice that by default the\n  `:cli_options` package is used for defining the CLI arguments schema.\n  - Implement any custom logic in your template's module.\n\nFor more details check the `Cascade.Template` docs.","ref":"readme.html#implementing-your-template"},{"type":"extras","title":"Installation - Overview","doc":"The package can be installed by adding `cascade` to your list of dependencies\nin `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:cascade, \"~> 0.1.0\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"License - Overview","doc":"Copyright (c) 2023 Panagiotis Nezis, Sportradar\n\nCascade is released under the MIT License. See the [LICENSE](LICENSE) file for more\ndetails.","ref":"readme.html#license"},{"type":"extras","title":"Changelog","doc":"# Changelog\n\nAll notable changes to this project will be documented in this file.\n\nThe format is based on [Keep a Changelog](https://keepachangelog.com/en/1.0.0/).\n\n## [v0.1.0](https://github.com/sportradar/elixir-workspace/tree/cascade/v0.1.0) (2024-05-13)\n\nInitial release.","ref":"changelog.html"},{"type":"extras","title":"License","doc":"# MIT License\n\nCopyright (c) 2023 Panagiotis Nezis, Sportradar\n\nPermission is hereby granted, free of charge, to any person obtaining\na copy of this software and associated documentation files (the\n\"Software\"), to deal in the Software without restriction, including\nwithout limitation the rights to use, copy, modify, merge, publish,\ndistribute, sublicense, and/or sell copies of the Software, and to\npermit persons to whom the Software is furnished to do so, subject to\nthe following conditions:\n\nThe above copyright notice and this permission notice shall be\nincluded in all copies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND,\nEXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF\nMERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND\nNONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE\nLIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION\nOF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION\nWITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.","ref":"license.html"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,50,46,50]}}